/*
 * unit.h
 *
 *  Created on: 2015-01-03
 *      Author: brendan
 *
 * http://www.jera.com/techinfo/jtns/jtn002.html - January 2015
 */

#ifndef UNIT_H_
#define UNIT_H_

#include <stdio.h>

unsigned int unit_asserts_passed, delta_unit_asserts_passed;
unsigned int unit_asserts_failed, delta_unit_asserts_failed;
unsigned int unit_tests_run;

#define TEST(name)		static void test_##name (void)

#define TEST_SUITE(suite) 	void all_tests_##suite (void)

#define ASSERT(message, test)	do {						\
				if( !(test) ) { 				\
					printf( "\t\tFAILURE: %s\n", message );	\
					fflush( stdout );			\
					++unit_asserts_failed;			\
				}						\
				else {						\
					++unit_asserts_passed;			\
				}						\
				} while(0)

#define ADD_TEST(name)		do {						\
				printf( "\tTest: %s...\n", #name );		\
				test_##name (); 				\
				++unit_tests_run;				\
				} while(0)

#define RUN_TEST_SUITE(suite)	do {							\
				printf("Running test suite: %s...\n", #suite);		\
				delta_unit_asserts_passed = unit_asserts_passed;	\
				delta_unit_asserts_failed = unit_asserts_failed;	\
				all_tests_##suite ();					\
				printf("Asserts passed: %d\nAsserts failed: %d\n\n",	\
					unit_asserts_passed - delta_unit_asserts_passed,\
					unit_asserts_failed - delta_unit_asserts_failed);\
				} while(0)

#define INIT_TESTING()			do {					\
						unit_asserts_passed = 0;	\
						delta_unit_asserts_passed = 0;	\
						unit_asserts_failed = 0;	\
						delta_unit_asserts_failed = 0;	\
						unit_tests_run = 0;		\
						printf("\n");			\
						fflush(stdout);			\
					} while( 0 )

#define PRINT_DIAG()			do {						\
						printf("DIAGNOSTICS...\n");		\
						printf(	"\tAsserts passed:\t%d\n"	\
							"\tAsserts failed:\t%d\n"	\
							"\tAsserts made:\t%d\n"		\
							"\tTests run:\t%d\n",		\
							unit_asserts_passed,		\
							unit_asserts_failed,		\
							unit_asserts_passed + unit_asserts_failed, \
							unit_tests_run );		\
					} while( 0 )


#endif /* UNIT_H_ */
